// -*- c++ -*-
/*
 *  MICO --- an Open Source CORBA implementation
 *  Copyright (c) 1997-2001 by The Mico Team
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the MICO Home Page at
 *  http://www.mico.org/
 */

#ifndef __mico_any_h__
#define __mico_any_h__

namespace CORBA {

class Any {
    typedef std::map<Long, Long, std::less<Long> > MapLL;

    TypeCodeChecker *checker;
    DataEncoder *ec;
    DataDecoder *dc;
    TypeCode_ptr thetc;
    DataEncoder::ValueState value_estate;
    DataDecoder::ValueState value_dstate;
    StaticAny *extracted_value;

    MICO_EXPORT TypeCode_ptr tc () const;
    MICO_EXPORT void tc (TypeCode_ptr);
    MICO_EXPORT void tc_if_changed (TypeCode_ptr);

    MICO_EXPORT void free ();
    MICO_EXPORT void copy (const Any &);
    MICO_EXPORT void reset_extracted_value ();

    MICO_EXPORT void prepare_write ();
    MICO_EXPORT void prepare_read ();
    MICO_EXPORT Boolean copy_any (Any &, MapLL &value_id_map, Boolean recurse = TRUE);
    MICO_EXPORT Boolean copy_any (Any &, Boolean recurse = TRUE);
    MICO_EXPORT Boolean compare_any (Any &,	MapLL &value_id_map);
    MICO_EXPORT Boolean compare_any (Any &);

    // dummy argument to distinguish from public Any ctor
    MICO_EXPORT Any (TypeCode_ptr tc, Buffer *, Boolean release, int dummy);
    MICO_EXPORT Any (TypeCode_ptr tc, DataDecoder *, DataEncoder *);

    MICO_EXPORT Boolean get_short (Short &);
    MICO_EXPORT Boolean get_ushort (UShort &);
    MICO_EXPORT Boolean get_long (Long &);
    MICO_EXPORT Boolean get_longlong (LongLong &);
    MICO_EXPORT Boolean get_ulong (ULong &);
    MICO_EXPORT Boolean get_ulonglong (ULongLong &);
    MICO_EXPORT Boolean get_float (Float &);
    MICO_EXPORT Boolean get_double (Double &);
    MICO_EXPORT Boolean get_longdouble (LongDouble &);
public:
    MICO_EXPORT Any ();
    MICO_EXPORT Any (const Any &);
    MICO_EXPORT Any (TypeCode_ptr tc, void *value, Boolean release = FALSE);
    MICO_EXPORT ~Any ();

    MICO_EXPORT Any &operator= (const Any &);
    // begin-mico-extension
    MICO_EXPORT Boolean operator== (const Any &) const;
    MICO_EXPORT Boolean equivalent (const Any &) const;
    // end-mico-extension

    struct from_boolean {
	from_boolean (Boolean b) : val (b) {}
	Boolean val;
    };
    struct from_octet {
	from_octet (Octet o) : val (o) {}
	Octet val;
    };
    struct from_char {
	from_char (Char c) : val (c) {}
	Char val;
    };
    struct from_wchar {
	from_wchar (WChar c) : val (c) {}
	WChar val;
    };
    struct from_string {
	from_string (char *s, ULong b, Boolean nc = FALSE)
	    : val (s), bound (b), nocopy (nc) {}
	from_string (const char *s, ULong b)
	    : val ((char *)s), bound (b), nocopy (FALSE) {}
	char *val;
	ULong bound;
	Boolean nocopy;
    };
    struct from_wstring {
	from_wstring (wchar_t *s, ULong b, Boolean nc = FALSE)
	    : val (s), bound (b), nocopy (nc) {}
	from_wstring (const wchar_t *s, ULong b)
	    : val ((wchar_t *)s), bound (b), nocopy (FALSE) {}
	wchar_t *val;
	ULong bound;
	Boolean nocopy;
    };
    struct from_fixed {
	from_fixed (const Fixed &f, UShort d, Short s)
            : fixed (f), digits (d), scale (s) {}
	const Fixed &fixed;
        UShort digits;
        Short scale;
    };
    // begin-mico-extension
    struct from_object {
	from_object (Object_ptr o, const char *n) : name (n), val (o) {}
	const char *name;
	Object_ptr val;
    };
    // end-mico-extension

    MICO_EXPORT void operator<<= (Short);
    MICO_EXPORT void operator<<= (Long);
    MICO_EXPORT void operator<<= (LongLong);
    MICO_EXPORT void operator<<= (UShort);
    MICO_EXPORT void operator<<= (ULong);
    MICO_EXPORT void operator<<= (ULongLong);
    MICO_EXPORT void operator<<= (Float);
    MICO_EXPORT void operator<<= (Double);
    MICO_EXPORT void operator<<= (LongDouble);
    MICO_EXPORT void operator<<= (const Any &);
    MICO_EXPORT void operator<<= (Any *);
    MICO_EXPORT void operator<<= (const Exception &);
    MICO_EXPORT void operator<<= (Exception *);
    MICO_EXPORT void operator<<= (const char *);
    MICO_EXPORT void operator<<= (const wchar_t *);
    MICO_EXPORT void operator<<= (from_boolean);
    MICO_EXPORT void operator<<= (from_octet);
    MICO_EXPORT void operator<<= (from_char);
    MICO_EXPORT void operator<<= (from_wchar);
    MICO_EXPORT void operator<<= (from_string);
    MICO_EXPORT void operator<<= (from_wstring);
    MICO_EXPORT void operator<<= (AbstractBase*);
    MICO_EXPORT void operator<<= (AbstractBase**);
    // begin-mico-extension
    MICO_EXPORT void operator<<= (from_fixed);
    MICO_EXPORT void operator<<= (from_object);
    MICO_EXPORT void operator<<= (Object_ptr);
    MICO_EXPORT void operator<<= (Object_ptr*);
    MICO_EXPORT void operator<<= (TypeCode_ptr);
    MICO_EXPORT void operator<<= (TypeCode_ptr*);
    MICO_EXPORT void operator<<= (Context_ptr);
    MICO_EXPORT void operator<<= (Context_ptr*);
    MICO_EXPORT void operator<<= (Principal_ptr);
    MICO_EXPORT void operator<<= (Principal_ptr*);

    MICO_EXPORT Boolean insert (Short);
    MICO_EXPORT Boolean insert (Long);
    MICO_EXPORT Boolean insert (LongLong);
    MICO_EXPORT Boolean insert (UShort);
    MICO_EXPORT Boolean insert (ULong);
    MICO_EXPORT Boolean insert (ULongLong);
    MICO_EXPORT Boolean insert (Float);
    MICO_EXPORT Boolean insert (Double);
    MICO_EXPORT Boolean insert (LongDouble);
    MICO_EXPORT Boolean insert (const Any &);
    MICO_EXPORT Boolean insert (Any *);
    MICO_EXPORT Boolean insert (const Exception &);
    MICO_EXPORT Boolean insert (Exception *);
    MICO_EXPORT Boolean insert (const char *);
    MICO_EXPORT Boolean insert (const wchar_t *);
    MICO_EXPORT Boolean insert (from_boolean);
    MICO_EXPORT Boolean insert (from_octet);
    MICO_EXPORT Boolean insert (from_char);
    MICO_EXPORT Boolean insert (from_wchar);
    MICO_EXPORT Boolean insert (from_string);
    MICO_EXPORT Boolean insert (from_wstring);
    MICO_EXPORT Boolean insert (from_fixed);
    MICO_EXPORT Boolean insert (from_object);
    MICO_EXPORT Boolean insert (Object_ptr);
    MICO_EXPORT Boolean insert (Object_ptr*);
    MICO_EXPORT Boolean insert (TypeCode_ptr);
    MICO_EXPORT Boolean insert (TypeCode_ptr*);
    MICO_EXPORT Boolean insert (Context_ptr);
    MICO_EXPORT Boolean insert (Context_ptr*);
    MICO_EXPORT Boolean insert (Principal_ptr);
    MICO_EXPORT Boolean insert (Principal_ptr*);
    MICO_EXPORT Boolean insert (AbstractBase*);
    MICO_EXPORT Boolean insert (AbstractBase**);
    // end-mico-extension

    struct to_boolean {
	to_boolean (Boolean &b) : ref (b) {}
	Boolean &ref;
    };
    struct to_octet {
	to_octet (Octet &o) : ref (o) {}
	Octet &ref;
    };
    struct to_char {
	to_char (Char &c) : ref (c) {}
	Char &ref;
    };
    struct to_wchar {
	to_wchar (WChar &c) : ref (c) {}
	WChar &ref;
    };
    struct to_string {
	to_string (const char * &s, ULong b) : ref (s), bound (b) {}
	const char * &ref;
	ULong bound;
    };
    struct to_wstring {
	to_wstring (const wchar_t * &s, ULong b) : ref (s), bound (b) {}
	const wchar_t * &ref;
	ULong bound;
    };
    struct to_object {
	to_object (Object_out o)
	    : ref (o.ptr()), name (0) {}
	to_object (Object_out o, String_out n)
	    : ref (o.ptr()), name (&n.ptr()) {}
	Object_ptr &ref;
	char **name;
    };
    struct to_fixed {
	to_fixed (Fixed &f, UShort d, Short s)
            : fixed (f), digits (d), scale (s) {}
	Fixed &fixed;
        UShort digits;
        Short scale;
    };
    struct to_abstract_base {
        to_abstract_base (AbstractBase *& base)
	  : ref (base) {}
        AbstractBase *& ref;
    };

    MICO_EXPORT Boolean operator>>= (Short &) const;
    MICO_EXPORT Boolean operator>>= (Long &) const;
    MICO_EXPORT Boolean operator>>= (LongLong &) const;
    MICO_EXPORT Boolean operator>>= (UShort &) const;
    MICO_EXPORT Boolean operator>>= (ULong &) const;
    MICO_EXPORT Boolean operator>>= (ULongLong &) const;
    MICO_EXPORT Boolean operator>>= (Float &) const;
    MICO_EXPORT Boolean operator>>= (Double &) const;
    MICO_EXPORT Boolean operator>>= (LongDouble &) const;
    MICO_EXPORT Boolean operator>>= (const Any *&) const;
    MICO_EXPORT Boolean operator>>= (const char * &) const;
    MICO_EXPORT Boolean operator>>= (const wchar_t * &) const;
    MICO_EXPORT Boolean operator>>= (to_boolean) const;
    MICO_EXPORT Boolean operator>>= (to_octet) const;
    MICO_EXPORT Boolean operator>>= (to_char) const;
    MICO_EXPORT Boolean operator>>= (to_wchar) const;
    MICO_EXPORT Boolean operator>>= (to_string) const;
    MICO_EXPORT Boolean operator>>= (to_wstring) const;
    MICO_EXPORT Boolean operator>>= (to_object) const;
    MICO_EXPORT Boolean operator>>= (to_abstract_base) const;
    // begin-mico-extension
    MICO_EXPORT Boolean operator>>= (to_fixed) const;
    MICO_EXPORT Boolean operator>>= (Object_ptr &) const;
    MICO_EXPORT Boolean operator>>= (TypeCode_ptr &) const;
    MICO_EXPORT Boolean operator>>= (Context_ptr &) const;
    MICO_EXPORT Boolean operator>>= (Principal_ptr &) const;
    // end-mico-extension

    MICO_EXPORT void replace (TypeCode_ptr tc, void *value, Boolean release = FALSE);

    // mico-extension
    MICO_EXPORT void replace (TypeCode_ptr tc);

    MICO_EXPORT const void *value () const;
    MICO_EXPORT TypeCode_ptr type () const;
    MICO_EXPORT void type (TypeCode_ptr);
    // begin-mico-extension
    MICO_EXPORT ULong length () const;
    MICO_EXPORT void set_type (TypeCode_ptr);
    MICO_EXPORT void reset ();
    MICO_EXPORT void rewind ();

    MICO_EXPORT void encode (DataEncoder &) const;
    MICO_EXPORT Boolean decode (DataDecoder &);

    MICO_EXPORT Boolean marshal (DataEncoder &);
    MICO_EXPORT Boolean demarshal (TypeCode_ptr, DataDecoder &);

    MICO_EXPORT Boolean from_static_any (const StaticAny &);
    MICO_EXPORT Boolean from_static_any (const StaticAny &, TypeCode_ptr);
    MICO_EXPORT Boolean to_static_any (StaticAny &) const;
    MICO_EXPORT Boolean to_static_any (StaticTypeInfo *ti, void *&value) const;
    MICO_EXPORT Boolean to_static_any (StaticTypeInfo *ti, CORBA::TypeCode_ptr,
			   void *&value) const;
    MICO_EXPORT Boolean to_static_any (StaticAny &, TypeCode_ptr) const;

    MICO_EXPORT Boolean coerce (CORBA::Any &dst) const;
    // end-mico-extension

    // begin-mico-extension
    // typesafe interface for constructed types
    MICO_EXPORT Boolean struct_put_begin ();
    MICO_EXPORT Boolean struct_put_end ();

    MICO_EXPORT Boolean union_put_begin ();
    MICO_EXPORT Boolean union_put_selection (Long idx);
    MICO_EXPORT Boolean union_put_end ();

    MICO_EXPORT Boolean enum_put (ULong val);
    MICO_EXPORT Boolean any_put (Any &, Boolean recurse = TRUE);

    MICO_EXPORT Boolean seq_put_begin (ULong len);
    MICO_EXPORT Boolean seq_put_end ();

    MICO_EXPORT Boolean array_put_begin ();
    MICO_EXPORT Boolean array_put_end ();

    MICO_EXPORT Boolean except_put_begin (const char *repoid);
    MICO_EXPORT Boolean except_put_end ();

    MICO_EXPORT Boolean value_put_begin (Long &value_id);
    MICO_EXPORT Boolean value_put_end (Long value_id);
    MICO_EXPORT Boolean value_put_ref (Long value_id);

    MICO_EXPORT Boolean valuebox_put_begin (Long &value_id);
    MICO_EXPORT Boolean valuebox_put_end (Long value_id);
    MICO_EXPORT Boolean valuebox_put_ref (Long value_id);


    MICO_EXPORT Boolean struct_get_begin () const;
    MICO_EXPORT Boolean struct_get_end () const;

    MICO_EXPORT Boolean union_get_begin () const;
    MICO_EXPORT Boolean union_get_selection (Long idx) const;
    MICO_EXPORT Boolean union_get_end () const;

    MICO_EXPORT Boolean enum_get (ULong &val) const;
    MICO_EXPORT Boolean any_get (Any &, Boolean recurse = TRUE) const;

    MICO_EXPORT Boolean seq_get_begin (ULong &len) const;
    MICO_EXPORT Boolean seq_get_end () const;

    MICO_EXPORT Boolean array_get_begin () const;
    MICO_EXPORT Boolean array_get_end () const;

    MICO_EXPORT Boolean except_get_begin (String_out repoid) const;
    MICO_EXPORT Boolean except_get_end () const;

    MICO_EXPORT Boolean value_get_begin (Long &value_id, Boolean &is_ref) const;
    MICO_EXPORT Boolean value_get_end (Long value_id, Boolean is_ref) const;

    MICO_EXPORT Boolean valuebox_get_begin (Long &value_id, Boolean &is_ref) const;
    MICO_EXPORT Boolean valuebox_get_end (Long value_id, Boolean is_ref) const;
    // end-mico-extension

private:
    void operator<<= (unsigned char) {}
    void operator>>= (unsigned char &) const {}
};

typedef TVarVar<Any> Any_var;
typedef TVarOut<Any> Any_out;

}

#endif // __mico_any_h__
